@@ -1,9 +1,9 @@
 package Net::DNS::RR::OPT;
 
 #
-# $Id: OPT.pm 1751 2019-07-21 19:55:12Z willem $
+# $Id: OPT.pm 1779 2020-05-11 09:11:17Z willem $
 #
-our $VERSION = (qw$LastChangedRevision: 1751 $)[1];
+our $VERSION = (qw$LastChangedRevision: 1779 $)[1];
 
 
 use strict;
@@ -116,12 +116,8 @@ sub version {
 
 sub size {
 	my $self = shift;
-	for ( $self->{size} ) {
-		my $UDP_size = 0;
-		( $UDP_size, $_ ) = ( shift || 0 ) if scalar @_;
-		return $UDP_size < 512 ? 512 : ( $_ = $UDP_size ) unless $_;
-		return $_ > 512 ? $_ : 512;
-	}
+	$self->{size} = shift if scalar @_;
+	return ( $self->{size} || 0 ) > 512 ? $self->{size} : 512;
 }
 
 
@@ -143,8 +139,8 @@ sub flags {
 
 sub options {
 	my ($self) = @_;
-	my $options = $self->{option} || {};
-	my @options = sort { $a <=> $b } keys %$options;
+	my $option = $self->{option} || {};
+	my @option = sort { $a <=> $b } keys %$option;
 }
 
 sub option {
@@ -190,7 +186,7 @@ sub _set_option {
 	my $options = $self->{option} ||= {};
 	delete $options->{$number};
 	return unless defined $value;
-	if ( ref($value) || scalar(@etc) ) {
+	if ( ref($value) || scalar(@etc) || $value !~ /\D/ ) {
 		my @arg = ( $value, @etc );
 		@arg = @$value if ref($value) eq 'ARRAY';
 		@arg = %$value if ref($value) eq 'HASH';
@@ -200,8 +196,11 @@ sub _set_option {
 			my $option  = ednsoptionbyval($number);
 			my $package = join '::', __PACKAGE__, $option;
 			$package =~ s/-/_/g;
-			croak "unable to compose option $option" unless $package->can('_compose');
-			$value = $package->_compose(@arg);
+			if ( $package->can('_compose') ) {
+				$value = $package->_compose(@arg);
+			} elsif ( scalar(@etc) ) {
+				croak "unable to compose option $option";
+			}
 		}
 	}
 	$options->{$number} = $value;
@@ -248,9 +247,8 @@ my @field8 = qw(FAMILY SOURCE-PREFIX-LENGTH SCOPE-PREFIX-LENGTH ADDRESS);
 sub _compose {
 	my ( $class, %argument ) = @_;
 	my $address = bless( {}, $family{$argument{FAMILY}} )->address( $argument{ADDRESS} );
-	my $preamble = pack 'nC2', map $_ ||= 0, @argument{@field8};
-	my $bitmask  = $argument{'SOURCE-PREFIX-LENGTH'};
-	pack "a* B$bitmask", $preamble, unpack 'B*', $address;
+	my $bitmask = $argument{'SOURCE-PREFIX-LENGTH'};
+	pack "a* B$bitmask", pack( 'nC2', map $_ || 0, @argument{@field8} ), unpack 'B*', $address;
 }
 
 sub _decompose {
@@ -362,6 +360,28 @@ sub _decompose {
 sub _image { &_decompose; }
 
 
+package Net::DNS::RR::OPT::EXTENDED_ERROR;			# draft-ietf-dnsop-extended-error
+use Net::DNS::Text;
+
+my @field15 = qw(INFO-CODE EXTRA-TEXT);
+
+sub _compose {
+	my ( $class, %argument ) = @_;
+	my ( $ic, $et ) = map $_ || '', @argument{@field15};
+	pack 'na*', $ic, Net::DNS::Text->new($et)->raw;
+}
+
+sub _decompose {
+	my ( $ic, $et ) = unpack 'na*', $_[1];
+	my @payload = ( 'INFO-CODE' => $ic, 'EXTRA-TEXT' => Net::DNS::Text->decode( \$et, 0, length $et )->value );
+}
+
+sub _image {
+	my %hash  = &_decompose;
+	my @image = map join( ' => ', $_, $hash{$_} ), @field15;
+}
+
+
 1;
 __END__
 
@@ -483,13 +503,12 @@ For some options, an array is more appropriate:
 	@algorithms = $packet->edns->option(6);
 
 
-Similar forms of array syntax may be used to construct the option value:
+Similar forms of array or hash syntax may be used to construct the
+option value:
 
 	$packet->edns->option( DHU => [1, 2, 4] );
-	$packet->edns->option( 6   => (1, 2, 4) );
 
 	$packet->edns->option( COOKIE => {'CLIENT-COOKIE' => $cookie} );
-	$packet->edns->option( 10     => ('CLIENT-COOKIE' => $cookie) );
 
 
 =head1 COPYRIGHT
