@@ -1,9 +1,9 @@
 package Net::DNS::Resolver::Base;
 
 #
-# $Id: Base.pm 1696 2018-07-20 16:15:11Z willem $
+# $Id: Base.pm 1781 2020-05-13 08:58:25Z willem $
 #
-our $VERSION = (qw$LastChangedRevision: 1696 $)[1];
+our $VERSION = (qw$LastChangedRevision: 1781 $)[1];
 
 
 #
@@ -26,7 +26,7 @@ our $VERSION = (qw$LastChangedRevision: 1696 $)[1];
 # [Revised March 2016, June 2018]
 
 
-use constant USE_SOCKET_IP => defined eval 'use IO::Socket::IP 0.32; 1;';
+use constant USE_SOCKET_IP => defined eval 'use IO::Socket::IP 0.38; 1';
 
 use constant IPv6 => USE_SOCKET_IP;
 
@@ -96,6 +96,18 @@ use constant PACKETSZ => 512;
 }
 
 
+my $warned;
+
+sub _deprecate {
+	carp join ' ', 'deprecated method;', pop(@_) unless $warned++;
+}
+
+
+sub _untaint {
+	return TAINT ? map ref($_) ? [_untaint(@$_)] : do { /^(.*)$/; $1 }, @_ : @_;
+}
+
+
 # These are the attributes that the user may specify in the new() constructor.
 my %public_attr = (
 	map( ( $_ => $_ ), keys %{&_defaults}, qw(domain nameserver srcaddr) ),
@@ -157,17 +169,12 @@ sub _option {
 }
 
 
-sub _untaint {
-	return TAINT ? map ref($_) ? [_untaint(@$_)] : do { /^(.*)$/; $1 }, @_ : @_;
-}
-
-
 sub _read_env {				## read resolver config environment variables
 	my $self = shift;
 
-	$self->nameservers( map split, $ENV{RES_NAMESERVERS} ) if defined $ENV{RES_NAMESERVERS};
+	$self->searchlist( map split, $ENV{LOCALDOMAIN} ) if defined $ENV{LOCALDOMAIN};
 
-	$self->domain( $ENV{LOCALDOMAIN} ) if defined $ENV{LOCALDOMAIN};
+	$self->nameservers( map split, $ENV{RES_NAMESERVERS} ) if defined $ENV{RES_NAMESERVERS};
 
 	$self->searchlist( map split, $ENV{RES_SEARCHLIST} ) if defined $ENV{RES_SEARCHLIST};
 
@@ -181,14 +188,14 @@ sub _read_config_file {			## read resolver config file
 	my $self = shift;
 	my $file = shift;
 
-	local *FILE;
-	open( FILE, $file ) or croak "$file: $!";
+	my $filehandle;
+	open( $filehandle, '<', $file ) or croak "$file: $!";
 
 	my @nameserver;
 	my @searchlist;
 
 	local $_;
-	while (<FILE>) {
+	while (<$filehandle>) {
 		s/[;#].*$//;					# strip comments
 
 		/^nameserver/ && do {
@@ -217,7 +224,7 @@ sub _read_config_file {			## read resolver config file
 		};
 	}
 
-	close(FILE);
+	close($filehandle);
 
 	$self->nameservers(@nameserver) if @nameserver;
 	$self->searchlist(@searchlist)	if @searchlist;
@@ -281,11 +288,11 @@ sub nameservers {
 			$defres->{persistent} = $self->{persistent};
 
 			my $names  = {};
-			my $packet = $defres->search( $ns, 'A' );
+			my $packet = $defres->send( $ns, 'A' );
 			my @iplist = _cname_addr( $packet, $names );
 
 			if (IPv6) {
-				$packet = $defres->search( $ns, 'AAAA' );
+				$packet = $defres->send( $ns, 'AAAA' );
 				push @iplist, _cname_addr( $packet, $names );
 			}
 
@@ -321,7 +328,7 @@ sub nameservers {
 	return @nameservers;
 }
 
-sub nameserver { &nameservers; }				# uncoverable pod
+sub nameserver { &nameservers; }
 
 sub _cname_addr {
 
@@ -340,12 +347,13 @@ sub _cname_addr {
 }
 
 
-sub answerfrom {
-	my $self = shift;
-	$self->{answerfrom} = shift if scalar @_;
-	return $self->{answerfrom};
+sub replyfrom {
+	return shift->{replyfrom};
 }
 
+sub answerfrom { &replyfrom; }					# uncoverable pod
+
+
 sub _reset_errorstring {
 	shift->{errorstring} = '';
 }
@@ -362,11 +370,7 @@ sub query {
 	my $self = shift;
 	my $name = shift || '.';
 
-	my @sfix;
-
-	if ( $self->{defnames} && ( ( $name =~ tr/././ ) < $self->{ndots} ) ) {
-		@sfix = $self->domain unless $name =~ m/:|\.\d*$/;
-	}
+	my @sfix = $self->{defnames} && ( $name !~ m/[.:]/ ) ? $self->domain : ();
 
 	my $fqdn = join '.', $name, @sfix;
 	$self->_diag( 'query(', $fqdn, @_, ')' );
@@ -381,15 +385,16 @@ sub search {
 	return $self->query(@_) unless $self->{dnsrch};
 
 	my $name = shift || '.';
+	my $dots = $name =~ tr/././;
 
-	my @sfix = ( $name =~ m/:|\.\d*$/ ) ? () : @{$self->{searchlist}};
-	my ( $domain, @etc ) = ( $name =~ tr/././ ) < $self->{ndots} ? (@sfix) : ( undef, @sfix );
+	my @sfix = ( $dots < $self->{ndots} ) ? @{$self->{searchlist}} : ();
+	my ( $one, @more ) = ( $name =~ m/:|\.\d*$/ ) ? () : ( $dots ? ( undef, @sfix ) : @sfix );
 
-	foreach my $suffix ( $domain, @etc ) {
+	foreach my $suffix ( $one, @more ) {
 		my $fqname = $suffix ? join( '.', $name, $suffix ) : $name;
 		$self->_diag( 'search(', $fqname, @_, ')' );
 		my $packet = $self->send( $fqname, @_ ) || next;
-		return $packet->header->ancount ? $packet : next;
+		return $packet if $packet->header->ancount;
 	}
 
 	return undef;
@@ -401,13 +406,15 @@ sub send {
 	my $packet	= $self->_make_query_packet(@_);
 	my $packet_data = $packet->data;
 
+	$self->_reset_errorstring;
+
 	return $self->_send_tcp( $packet, $packet_data )
 			if $self->{usevc} || length $packet_data > $self->_packetsz;
 
-	my $ans = $self->_send_udp( $packet, $packet_data ) || return;
+	my $reply = $self->_send_udp( $packet, $packet_data ) || return;
 
-	return $ans if $self->{igntc};
-	return $ans unless $ans->header->tc;
+	return $reply if $self->{igntc};
+	return $reply unless $reply->header->tc;
 
 	$self->_diag('packet truncated: retrying using TCP');
 	$self->_send_tcp( $packet, $packet_data );
@@ -417,64 +424,59 @@ sub send {
 sub _send_tcp {
 	my ( $self, $query, $query_data ) = @_;
 
-	$self->_reset_errorstring;
-
 	my $tcp_packet = pack 'n a*', length($query_data), $query_data;
 	my @ns = $self->nameservers();
-	my $lastanswer;
+	my $fallback;
 	my $timeout = $self->{tcp_timeout};
 
 	foreach my $ip (@ns) {
-		my $socket = $self->_create_tcp_socket($ip) || next;
-		my $select = IO::Select->new($socket);
-
 		$self->_diag( 'tcp send', "[$ip]" );
 
+		my $socket = $self->_create_tcp_socket($ip);
+		$self->errorstring($!);
+		my $select = IO::Select->new( $socket || next );
+
 		$socket->send($tcp_packet);
 		$self->errorstring($!);
 
-		next unless $select->can_read($timeout);	# uncoverable branch
+		next unless $select->can_read($timeout);	# uncoverable branch true
 
 		my $buffer = _read_tcp($socket);
-		$self->answerfrom($ip);
-		$self->_diag( 'answer from', "[$ip]", length($buffer), 'bytes' );
+		$self->{replyfrom} = $ip;
+		$self->_diag( 'reply from', "[$ip]", length($buffer), 'bytes' );
 
 		my $reply = Net::DNS::Packet->decode( \$buffer, $self->{debug} );
 		$self->errorstring($@);
 		next unless $self->_accept_reply( $reply, $query );
-		$reply->answerfrom($ip);
+		$reply->from($ip);
 
 		if ( $self->{tsig_rr} && !$reply->verify($query) ) {
 			$self->errorstring( $reply->verifyerr );
 			next;
 		}
 
-		$lastanswer = $reply;
-
 		my $rcode = $reply->header->rcode;
-		$self->errorstring($rcode);			# historical quirk
 		return $reply if $rcode eq 'NOERROR';
 		return $reply if $rcode eq 'NXDOMAIN';
+		$fallback = $reply;
 	}
 
-	$self->{errorstring} = $lastanswer->header->rcode if $lastanswer;
+	$self->{errorstring} = $fallback->header->rcode if $fallback;
 	$self->errorstring('query timed out') unless $self->{errorstring};
-	return $lastanswer;
+	return $fallback;
 }
 
 
 sub _send_udp {
 	my ( $self, $query, $query_data ) = @_;
 
-	$self->_reset_errorstring;
-
 	my @ns	    = $self->nameservers;
 	my $port    = $self->{port};
 	my $retrans = $self->{retrans} || 1;
 	my $retry   = $self->{retry} || 1;
 	my $servers = scalar(@ns);
 	my $timeout = $servers ? do { no integer; $retrans / $servers } : 0;
-	my $lastanswer;
+	my $fallback;
 
 	# Perform each round of retries.
 RETRY: for ( 1 .. $retry ) {					# assumed to be a small number
@@ -486,8 +488,8 @@ NAMESERVER: foreach my $ns (@ns) {
 
 			# state vector replaces corresponding element of @ns array
 			unless ( ref $ns ) {
-				my $socket = $self->_create_udp_socket($ns) || next;
 				my $dst_sockaddr = $self->_create_dst_sockaddr( $ns, $port );
+				my $socket = $self->_create_udp_socket($ns) || next;
 				$ns = [$socket, $ns, $dst_sockaddr];
 			}
 
@@ -507,16 +509,16 @@ NAMESERVER: foreach my $ns (@ns) {
 			my $reply;
 			while ( my ($socket) = $select->can_read($timeout) ) {
 				my $peer = $socket->peerhost;
-				$self->answerfrom($peer);
+				$self->{replyfrom} = $peer;
 
 				my $buffer = _read_udp( $socket, $self->_packetsz );
-				$self->_diag( "answer from [$peer]", length($buffer), 'bytes' );
+				$self->_diag( "reply from [$peer]", length($buffer), 'bytes' );
 
 				my $packet = Net::DNS::Packet->decode( \$buffer, $self->{debug} );
 				$self->errorstring($@);
 				next unless $self->_accept_reply( $packet, $query );
 				$reply = $packet;
-				$reply->answerfrom($peer);
+				$reply->from($peer);
 				last;
 			}					#SELECT LOOP
 
@@ -527,12 +529,10 @@ NAMESERVER: foreach my $ns (@ns) {
 				next;
 			}
 
-			$lastanswer = $reply;
-
 			my $rcode = $reply->header->rcode;
-			$self->errorstring($rcode);		# historical quirk
 			return $reply if $rcode eq 'NOERROR';
 			return $reply if $rcode eq 'NXDOMAIN';
+			$fallback = $reply;
 			$$ns[3] = $rcode;
 		}						#NAMESERVER LOOP
 
@@ -540,9 +540,9 @@ NAMESERVER: foreach my $ns (@ns) {
 		$timeout += $timeout;
 	}							#RETRY LOOP
 
-	$self->{errorstring} = $lastanswer->header->rcode if $lastanswer;
+	$self->{errorstring} = $fallback->header->rcode if $fallback;
 	$self->errorstring('query timed out') unless $self->{errorstring};
-	return $lastanswer;
+	return $fallback;
 }
 
 
@@ -551,6 +551,8 @@ sub bgsend {
 	my $packet	= $self->_make_query_packet(@_);
 	my $packet_data = $packet->data;
 
+	$self->_reset_errorstring;
+
 	return $self->_bgsend_tcp( $packet, $packet_data )
 			if $self->{usevc} || length $packet_data > $self->_packetsz;
 
@@ -561,18 +563,19 @@ sub bgsend {
 sub _bgsend_tcp {
 	my ( $self, $packet, $packet_data ) = @_;
 
-	$self->_reset_errorstring;
-
 	my $tcp_packet = pack 'n a*', length($packet_data), $packet_data;
 
 	foreach my $ip ( $self->nameservers ) {
-		my $socket = $self->_create_tcp_socket($ip) || next;
-
 		$self->_diag( 'bgsend', "[$ip]" );
 
+		my $socket = $self->_create_tcp_socket($ip);
+		$self->errorstring($!);
+		next unless $socket;
+
 		$socket->blocking(0);
 		$socket->send($tcp_packet);
 		$self->errorstring($!);
+		$socket->blocking(1);
 
 		my $expire = time() + $self->{tcp_timeout};
 		${*$socket}{net_dns_bg} = [$expire, $packet];
@@ -586,13 +589,11 @@ sub _bgsend_tcp {
 sub _bgsend_udp {
 	my ( $self, $packet, $packet_data ) = @_;
 
-	$self->_reset_errorstring;
-
 	my $port = $self->{port};
 
 	foreach my $ip ( $self->nameservers ) {
-		my $socket = $self->_create_udp_socket($ip) || next;
 		my $dst_sockaddr = $self->_create_dst_sockaddr( $ip, $port );
+		my $socket = $self->_create_udp_socket($ip) || next;
 
 		$self->_diag( 'bgsend', "[$ip]:$port" );
 
@@ -638,13 +639,14 @@ sub bgbusy {
 
 
 sub bgisready {				## historical
-	!&bgbusy;						# uncoverable pod
+	_deprecate('prefer  ! bgbusy(...)');			# uncoverable pod
+	!&bgbusy;
 }
 
 
 sub bgread {
 	while (&bgbusy) {					# side effect: TCP retry
-		IO::Select->new( $_[1] )->can_read(0.02);	# use 3 orders of magnitude less CPU
+		IO::Select->new( $_[1] )->can_read(0.02);	# reduce my CPU usage by 3 orders of magnitude
 	}
 	&_bgread;
 }
@@ -658,22 +660,22 @@ sub _bgread {
 	my ( $expire, $query, $read ) = @$appendix;
 	return shift(@$read) if ref($read);
 
-	unless ( IO::Select->new($handle)->can_read(0) ) {
+	my $select = IO::Select->new($handle);
+	unless ( $select->can_read(0) ) {
 		$self->errorstring('timed out');
 		return;
 	}
 
-	my $peer = $handle->peerhost;
-	$self->answerfrom($peer);
+	my $peer = $self->{replyfrom} = $handle->peerhost;
 
 	my $dgram = $handle->socktype() == SOCK_DGRAM;
 	my $buffer = $dgram ? _read_udp( $handle, $self->_packetsz ) : _read_tcp($handle);
-	$self->_diag( "answer from [$peer]", length($buffer), 'bytes' );
+	$self->_diag( "reply from [$peer]", length($buffer), 'bytes' );
 
 	my $reply = Net::DNS::Packet->decode( \$buffer, $self->{debug} );
 	$self->errorstring($@);
 	return unless $self->_accept_reply( $reply, $query );
-	$reply->answerfrom($peer);
+	$reply->from($peer);
 
 	return $reply unless $self->{tsig_rr} && !$reply->verify($query);
 	$self->errorstring( $reply->verifyerr );
@@ -689,8 +691,9 @@ sub _accept_reply {
 	my $header = $reply->header;
 	return unless $header->qr;
 
-	return 1 unless $query;					# SpamAssassin 3.4.1 workaround
-	return $header->id == $query->header->id;
+	return if $query && $header->id != $query->header->id;
+
+	$self->errorstring( $header->rcode );			# historical quirk
 }
 
 
@@ -713,11 +716,12 @@ sub axfr {				## zone transfer
 				$soa = $rr;
 			}
 
-			return $rr if scalar @rr;
+			unless ( scalar @rr ) {
+				my $reply;			# refill @rr
+				( $reply, $verify ) = $self->_axfr_next( $select, $verify );
+				@rr = $reply->answer;
+			}
 
-			my $reply;
-			( $reply, $verify ) = $self->_axfr_next( $select, $verify );
-			@rr = $reply->answer;
 			return $rr;
 		};
 
@@ -734,13 +738,15 @@ sub axfr {				## zone transfer
 
 
 sub axfr_start {			## historical
-	my $self = shift;					# uncoverable pod
+	_deprecate('prefer  $iterator = $self->axfr(...)');	# uncoverable pod
+	my $self = shift;
 	defined( $self->{axfr_iter} = $self->axfr(@_) );
 }
 
 
 sub axfr_next {				## historical
-	shift->{axfr_iter}->();					# uncoverable pod
+	_deprecate('prefer  $iterator->()');			# uncoverable pod
+	shift->{axfr_iter}->();
 }
 
 
@@ -753,15 +759,16 @@ sub _axfr_start {
 	my $content = $request->data;
 	my $TCP_msg = pack 'n a*', length($content), $content;
 
-	$self->_diag("axfr_start( $dname @class )");
+	$self->_diag("axfr( $dname @class )");
 
 	my ( $select, $reply, $rcode );
 	foreach my $ns ( $self->nameservers ) {
-		my $socket = $self->_create_tcp_socket($ns) || next;
+		$self->_diag("axfr send [$ns]");
 
-		$self->_diag("axfr_start nameserver [$ns]");
+		my $socket = $self->_create_tcp_socket($ns);
+		$self->errorstring($!);
+		$select = IO::Select->new( $socket || next );
 
-		$select = IO::Select->new($socket);
 		$socket->send($TCP_msg);
 		$self->errorstring($!);
 
@@ -794,7 +801,7 @@ sub _axfr_next {
 	my ($socket) = $select->can_read( $self->{tcp_timeout} );
 	croak $self->errorstring('timed out') unless $socket;
 
-	$self->answerfrom( $socket->peerhost );
+	$self->{replyfrom} = $socket->peerhost;
 
 	my $buffer = _read_tcp($socket);
 	$self->_diag( 'received', length($buffer), 'bytes' );
@@ -816,22 +823,17 @@ sub _axfr_next {
 sub _read_tcp {
 	my $socket = shift;
 
-	my ( $s1, $s2 );
+	my ( $buffer, $s1, $s2 );
 	$socket->recv( $s1, 2 );				# one lump
 	$socket->recv( $s2, 2 - length $s1 );			# or two?
 	my $size = unpack 'n', pack( 'a*a*@2', $s1, $s2 );
 
-	my $buffer = '';
-	while ( ( my $read = length $buffer ) < $size ) {
+	$socket->recv( $buffer, $size );			# initial read
 
-		# During some of my tests recv() returned undef even
-		# though there was no error.  Checking the amount
-		# of data read appears to work around that problem.
-
-		my $recv_buf;
-		$socket->recv( $recv_buf, $size - $read );
-
-		$buffer .= $recv_buf || last;
+	while ( length($buffer) < $size ) {
+		my $fragment;
+		$socket->recv( $fragment, $size - length($buffer) );
+		$buffer .= $fragment || last;
 	}
 	return $buffer;
 }
@@ -885,7 +887,6 @@ sub _create_tcp_socket {
 				unless $ip6_addr;
 	}
 
-	$self->errorstring("no socket $sock_key $!") unless $socket;
 	$self->{persistent}{$sock_key} = $self->{persistent_tcp} ? $socket : undef;
 	return $socket;
 }
@@ -918,7 +919,6 @@ sub _create_udp_socket {
 				unless $ip6_addr;
 	}
 
-	$self->errorstring("no socket $sock_key $!") unless $socket;
 	$self->{persistent}{$sock_key} = $self->{persistent_udp} ? $socket : undef;
 	return $socket;
 }
@@ -1009,25 +1009,25 @@ sub dnssec {
 
 sub force_v6 {
 	my $self = shift;
-	my $value = scalar(@_) ? shift() : $self->{force_v6};
+	my $value = scalar(@_) ? $_[0] : $self->{force_v6};
 	$self->{force_v6} = $value ? do { $self->{force_v4} = 0; 1 } : 0;
 }
 
 sub force_v4 {
 	my $self = shift;
-	my $value = scalar(@_) ? shift() : $self->{force_v4};
+	my $value = scalar(@_) ? $_[0] : $self->{force_v4};
 	$self->{force_v4} = $value ? do { $self->{force_v6} = 0; 1 } : 0;
 }
 
 sub prefer_v6 {
 	my $self = shift;
-	my $value = scalar(@_) ? shift() : $self->{prefer_v6};
+	my $value = scalar(@_) ? $_[0] : $self->{prefer_v6};
 	$self->{prefer_v6} = $value ? do { $self->{prefer_v4} = 0; 1 } : 0;
 }
 
 sub prefer_v4 {
 	my $self = shift;
-	my $value = scalar(@_) ? shift() : $self->{prefer_v4};
+	my $value = scalar(@_) ? $_[0] : $self->{prefer_v4};
 	$self->{prefer_v4} = $value ? do { $self->{prefer_v6} = 0; 1 } : 0;
 }
 
@@ -1071,13 +1071,8 @@ sub udppacketsize {
 #
 # Keep this method around. Folk depend on it although it is neither documented nor exported.
 #
-my $warned;
-
 sub make_query_packet {			## historical
-	unless ( $warned++ ) {					# uncoverable pod
-		local $SIG{__WARN__};
-		carp 'deprecated method; see RT#37104';
-	}
+	_deprecate('see RT#37104');				# uncoverable pod
 	&_make_query_packet;
 }
 
@@ -1088,6 +1083,33 @@ sub _diag {				## debug output
 }
 
 
+{
+	my $parse_dig = sub {
+		require Net::DNS::ZoneFile;
+
+		my $dug = new Net::DNS::ZoneFile( \*DATA );
+		my @rr	= $dug->read;
+
+		my @auth = grep $_->type eq 'NS', @rr;
+		my %auth = map { lc $_->nsdname => 1 } @auth;
+		my %glue;
+		my @glue = grep $auth{lc $_->name}, @rr;
+		foreach ( grep $_->can('address'), @glue ) {
+			push @{$glue{lc $_->name}}, $_->address;
+		}
+		map @$_, values %glue;
+	};
+
+	my @ip;
+
+	sub _hints {			## default hints
+		@ip = &$parse_dig unless scalar @ip;		# once only, on demand
+		splice @ip, 0, 0, splice( @ip, int( rand scalar @ip ) );    # cut deck
+		return @ip;
+	}
+}
+
+
 our $AUTOLOAD;
 
 sub DESTROY { }				## Avoid tickling AUTOLOAD (in cleanup)
@@ -1097,7 +1119,7 @@ sub AUTOLOAD {				## Default method
 
 	my $name = $AUTOLOAD;
 	$name =~ s/.*://;
-	croak "$name: no such method" unless $public_attr{$name};
+	croak qq[unknown method "$name"] unless $public_attr{$name};
 
 	no strict q/refs/;
 	*{$AUTOLOAD} = sub {
@@ -1113,8 +1135,6 @@ sub AUTOLOAD {				## Default method
 
 1;
 
-__END__
-
 
 =head1 NAME
 
@@ -1135,13 +1155,15 @@ for all your resolving needs.
 
 =head1 METHODS
 
-=head2 new, domain, searchlist, nameservers, print, string, errorstring,
+=head2 new, domain, searchlist, nameserver, nameservers,
 
-=head2 search, query, send, bgsend, bgbusy, bgread, axfr, answerfrom,
+=head2 search, query, send, bgsend, bgbusy, bgread, axfr,
 
 =head2 force_v4, force_v6, prefer_v4, prefer_v6,
 
-=head2 dnssec, srcaddr, tsig, udppacketsize
+=head2 dnssec, srcaddr, tsig, udppacketsize,
+
+=head2 print, string, errorstring, replyfrom
 
 See L<Net::DNS::Resolver>.
 
@@ -1182,3 +1204,69 @@ L<perl>, L<Net::DNS>, L<Net::DNS::Resolver>
 
 =cut
 
+
+########################################
+
+__DATA__	## DEFAULT HINTS
+
+; <<>> DiG 9.11.4-RedHat-9.11.4-4.fc28 <<>> @b.root-servers.net . -t NS
+; (2 servers found)
+;; global options: +cmd
+;; Got answer:
+;; ->>HEADER<<- opcode: QUERY, status: NOERROR, id: 44111
+;; flags: qr aa rd; QUERY: 1, ANSWER: 13, AUTHORITY: 0, ADDITIONAL: 27
+;; WARNING: recursion requested but not available
+
+;; OPT PSEUDOSECTION:
+; EDNS: version: 0, flags:; udp: 4096
+;; QUESTION SECTION:
+;.				IN	NS
+
+;; ANSWER SECTION:
+.			518400	IN	NS	c.root-servers.net.
+.			518400	IN	NS	k.root-servers.net.
+.			518400	IN	NS	l.root-servers.net.
+.			518400	IN	NS	j.root-servers.net.
+.			518400	IN	NS	b.root-servers.net.
+.			518400	IN	NS	g.root-servers.net.
+.			518400	IN	NS	h.root-servers.net.
+.			518400	IN	NS	d.root-servers.net.
+.			518400	IN	NS	a.root-servers.net.
+.			518400	IN	NS	f.root-servers.net.
+.			518400	IN	NS	i.root-servers.net.
+.			518400	IN	NS	m.root-servers.net.
+.			518400	IN	NS	e.root-servers.net.
+
+;; ADDITIONAL SECTION:
+a.root-servers.net.	3600000	IN	A	198.41.0.4
+b.root-servers.net.	3600000	IN	A	199.9.14.201
+c.root-servers.net.	3600000	IN	A	192.33.4.12
+d.root-servers.net.	3600000	IN	A	199.7.91.13
+e.root-servers.net.	3600000	IN	A	192.203.230.10
+f.root-servers.net.	3600000	IN	A	192.5.5.241
+g.root-servers.net.	3600000	IN	A	192.112.36.4
+h.root-servers.net.	3600000	IN	A	198.97.190.53
+i.root-servers.net.	3600000	IN	A	192.36.148.17
+j.root-servers.net.	3600000	IN	A	192.58.128.30
+k.root-servers.net.	3600000	IN	A	193.0.14.129
+l.root-servers.net.	3600000	IN	A	199.7.83.42
+m.root-servers.net.	3600000	IN	A	202.12.27.33
+a.root-servers.net.	3600000	IN	AAAA	2001:503:ba3e::2:30
+b.root-servers.net.	3600000	IN	AAAA	2001:500:200::b
+c.root-servers.net.	3600000	IN	AAAA	2001:500:2::c
+d.root-servers.net.	3600000	IN	AAAA	2001:500:2d::d
+e.root-servers.net.	3600000	IN	AAAA	2001:500:a8::e
+f.root-servers.net.	3600000	IN	AAAA	2001:500:2f::f
+g.root-servers.net.	3600000	IN	AAAA	2001:500:12::d0d
+h.root-servers.net.	3600000	IN	AAAA	2001:500:1::53
+i.root-servers.net.	3600000	IN	AAAA	2001:7fe::53
+j.root-servers.net.	3600000	IN	AAAA	2001:503:c27::2:30
+k.root-servers.net.	3600000	IN	AAAA	2001:7fd::1
+l.root-servers.net.	3600000	IN	AAAA	2001:500:9f::42
+m.root-servers.net.	3600000	IN	AAAA	2001:dc3::35
+
+;; Query time: 173 msec
+;; SERVER: 199.9.14.201#53(199.9.14.201)
+;; WHEN: Fri Aug 10 19:03:11 BST 2018
+;; MSG SIZE  rcvd: 811
+
